# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from pathlib import Path

import pytest

import botocore.exceptions
from botocore.config import Config
from botocore.session import get_session

_SDK_DEFAULT_CONFIGURATION_VALUES_ALLOWLIST = (
    'retryMode',
    'stsRegionalEndpoints',
    's3UsEast1RegionalEndpoints',
    'connectTimeoutInMillis',
    'tlsNegotiationTimeoutInMillis',
)

session = get_session()
loader = session.get_component('data_loader')
sdk_default_configuration = loader.load_data('sdk-default-configuration')


def assert_client_uses_standard_defaults(client):
    assert client.meta.config.s3['us_east_1_regional_endpoint'] == 'regional'
    assert client.meta.config.connect_timeout == 3.1
    assert client.meta.endpoint_url == 'https://sts.us-west-2.amazonaws.com'
    assert client.meta.config.retries['mode'] == 'standard'


@pytest.mark.parametrize("mode", sdk_default_configuration['base'])
def test_no_new_sdk_default_configuration_values(mode):
    err_msg = (
        f'New default configuration value {mode} introduced to '
        f'sdk-default-configuration.json. Support for setting {mode} must be '
        'considered and added to the DefaulConfigResolver. In addition, '
        'must add value to _SDK_DEFAULT_CONFIGURATION_VALUES_ALLOWLIST.'
    )
    assert mode in _SDK_DEFAULT_CONFIGURATION_VALUES_ALLOWLIST, err_msg


def test_default_configurations_resolve_correctly():
    session = get_session()
    config = Config(defaults_mode='standard')
    client = session.create_client(
        'sts', config=config, region_name='us-west-2'
    )
    assert_client_uses_standard_defaults(client)


@pytest.fixture
def loader():
    test_models_dir = Path(__file__).parent / 'models'
    loader = botocore.loaders.Loader()
    loader.search_paths.insert(0, test_models_dir)
    return loader


@pytest.fixture
def session(loader):
    session = botocore.session.Session()
    session.register_component('data_loader', loader)
    return session


def assert_client_uses_legacy_defaults(client):
    assert client.meta.config.s3 is None
    assert client.meta.config.connect_timeout == 60
    assert client.meta.endpoint_url == 'https://sts.amazonaws.com'
    assert client.meta.config.retries['mode'] == 'legacy'


def assert_client_uses_testing_defaults(client):
    assert client.meta.config.s3['us_east_1_regional_endpoint'] == 'regional'
    assert client.meta.config.connect_timeout == 9999
    assert client.meta.endpoint_url == 'https://sts.amazonaws.com'
    assert client.meta.config.retries['mode'] == 'standard'


class TestConfigurationDefaults:
    def test_defaults_mode_resolved_from_config_store(self, session):
        config_store = session.get_component('config_store')
        config_store.set_config_variable('defaults_mode', 'standard')
        client = session.create_client('sts', 'us-west-2')
        assert_client_uses_testing_defaults(client)

    def test_no_mutate_session_provider(self, session):
        # Using the standard default mode should change the connect timeout
        # on the client, but not the session
        standard_client = session.create_client(
            'sts', 'us-west-2', config=Config(defaults_mode='standard')
        )
        assert_client_uses_testing_defaults(standard_client)

        # Using the legacy default mode should not change the connect timeout
        # on the client or the session. By default the connect timeout for a client
        # is 60 seconds, and unset on the session.
        legacy_client = session.create_client('sts', 'us-west-2')
        assert_client_uses_legacy_defaults(legacy_client)

    def test_defaults_mode_resolved_from_client_config(self, session):
        config = Config(defaults_mode='standard')
        client = session.create_client('sts', 'us-west-2', config=config)
        assert_client_uses_testing_defaults(client)

    def test_defaults_mode_resolved_invalid_mode_exception(self, session):
        with pytest.raises(botocore.exceptions.InvalidDefaultsMode):
            config = Config(defaults_mode='invalid_default_mode')
            session.create_client('sts', 'us-west-2', config=config)

    def test_defaults_mode_resolved_legacy(self, session):
        client = session.create_client('sts', 'us-west-2')
        assert_client_uses_legacy_defaults(client)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import time

import botocore.session
from botocore import exceptions
from tests import unittest


class TestApigateway(unittest.TestCase):
    def setUp(self):
        self.session = botocore.session.get_session()
        self.client = self.session.create_client('apigateway', 'us-east-1')

        # Create a resource to use with this client.
        self.api_name = 'mytestapi'
        self.api_id = self.create_rest_api_or_skip()

    def create_rest_api_or_skip(self):
        try:
            api_id = self.client.create_rest_api(name=self.api_name)['id']
        except exceptions.ClientError as e:
            if e.response['Error']['Code'] == 'TooManyRequestsException':
                raise unittest.SkipTest(
                    "Hit API gateway throttle limit, skipping test."
                )
            raise
        return api_id

    def delete_api(self):
        retries = 0
        while retries < 10:
            try:
                self.client.delete_rest_api(restApiId=self.api_id)
                break
            except exceptions.ClientError as e:
                if e.response['Error']['Code'] == 'TooManyRequestsException':
                    retries += 1
                    time.sleep(5)
                else:
                    raise

    def tearDown(self):
        self.delete_api()

    def test_put_integration(self):
        # The only resource on a brand new api is the path. So use that ID.
        path_resource_id = self.client.get_resources(restApiId=self.api_id)[
            'items'
        ][0]['id']

        # Create a method for the resource.
        self.client.put_method(
            restApiId=self.api_id,
            resourceId=path_resource_id,
            httpMethod='GET',
            authorizationType='None',
        )

        # Put an integration on the method.
        response = self.client.put_integration(
            restApiId=self.api_id,
            resourceId=path_resource_id,
            httpMethod='GET',
            type='HTTP',
            integrationHttpMethod='GET',
            uri='https://api.endpoint.com',
        )
        # Assert the response was successful by checking the integration type
        self.assertEqual(response['type'], 'HTTP')



# Copyright 2012-2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore import translate
from tests import unittest


class TestBuildRetryConfig(unittest.TestCase):
    def setUp(self):
        self.retry = {
            "definitions": {"def_name": {"from": {"definition": "file"}}},
            "retry": {
                "__default__": {
                    "max_attempts": 5,
                    "delay": "global_delay",
                    "policies": {
                        "global_one": "global",
                        "override_me": "global",
                    },
                },
                "sts": {
                    "__default__": {
                        "delay": "service_specific_delay",
                        "policies": {
                            "service_one": "service",
                            "override_me": "service",
                        },
                    },
                    "AssumeRole": {
                        "policies": {
                            "name": "policy",
                            "other": {"$ref": "def_name"},
                        }
                    },
                },
            },
        }

    def test_inject_retry_config(self):
        retry = translate.build_retry_config(
            'sts', self.retry['retry'], self.retry['definitions']
        )
        self.assertIn('__default__', retry)
        self.assertEqual(
            retry['__default__'],
            {
                "max_attempts": 5,
                "delay": "service_specific_delay",
                "policies": {
                    "global_one": "global",
                    "override_me": "service",
                    "service_one": "service",
                },
            },
        )
        # Policies should be merged.
        operation_config = retry['AssumeRole']
        self.assertEqual(operation_config['policies']['name'], 'policy')

    def test_resolve_reference(self):
        retry = translate.build_retry_config(
            'sts', self.retry['retry'], self.retry['definitions']
        )
        operation_config = retry['AssumeRole']
        # And we should resolve references.
        self.assertEqual(
            operation_config['policies']['other'],
            {"from": {"definition": "file"}},
        )

    def test_service_specific_defaults_no_mutate_default_retry(self):
        retry = translate.build_retry_config(
            'sts', self.retry['retry'], self.retry['definitions']
        )
        # sts has a specific policy
        self.assertEqual(
            retry['__default__'],
            {
                "max_attempts": 5,
                "delay": "service_specific_delay",
                "policies": {
                    "global_one": "global",
                    "override_me": "service",
                    "service_one": "service",
                },
            },
        )

        # The general defaults for the upstream model should not have been
        # mutated from building the retry config
        self.assertEqual(
            self.retry['retry']['__default__'],
            {
                "max_attempts": 5,
                "delay": "global_delay",
                "policies": {
                    "global_one": "global",
                    "override_me": "global",
                },
            },
        )

    def test_client_override_max_attempts(self):
        retry = translate.build_retry_config(
            'sts',
            self.retry['retry'],
            self.retry['definitions'],
            client_retry_config={'max_attempts': 9},
        )
        self.assertEqual(retry['__default__']['max_attempts'], 10)
        # But it should not mutate the original retry model
        self.assertEqual(self.retry['retry']['__default__']['max_attempts'], 5)



import random
import threading
import time

from botocore.retries import quota
from tests import unittest


class TestRetryQuota(unittest.TestCase):
    def setUp(self):
        self.max_capacity = 50
        self.retry_quota = quota.RetryQuota(self.max_capacity)
        self.shutdown_threads = False
        self.seen_capacities = []

    def run_in_thread(self):
        while not self.shutdown_threads:
            capacity = random.randint(1, self.max_capacity)
            self.retry_quota.acquire(capacity)
            self.seen_capacities.append(self.retry_quota.available_capacity)
            self.retry_quota.release(capacity)
            self.seen_capacities.append(self.retry_quota.available_capacity)

    def test_capacity_stays_within_range(self):
        # The main thing we're after is that the available_capacity
        # should always be 0 <= capacity <= max_capacity.
        # So what we do is spawn a number of threads and them acquire
        # random capacity.  They'll check that they never see an invalid
        # capacity.
        threads = []
        for i in range(10):
            threads.append(threading.Thread(target=self.run_in_thread))
        for thread in threads:
            thread.start()
        # We'll let things run for a second.
        time.sleep(1)
        self.shutdown_threads = True
        for thread in threads:
            thread.join()
        for seen_capacity in self.seen_capacities:
            self.assertGreaterEqual(seen_capacity, 0)
            self.assertLessEqual(seen_capacity, self.max_capacity)



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import re
from contextlib import contextmanager

import pytest

from botocore import exceptions
from botocore.compat import urlsplit
from botocore.config import Config
from botocore.exceptions import (
    InvalidHostLabelError,
    ParamValidationError,
    UnsupportedS3ControlArnError,
)
from botocore.session import Session
from tests import ClientHTTPStubber, unittest

ACCESSPOINT_ARN_TEST_CASES = [
    # Outpost accesspoint arn test cases
    {
        'arn': (
            'arn:aws:s3-outposts:us-west-2:123456789012:'
            'outpost:op-01234567890123456:accesspoint:myaccesspoint'
        ),
        'region': 'us-west-2',
        'config': {},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-west-2.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'region': 'us-west-2',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'region': 'us-west-2',
        'config': {'s3': {'use_arn_region': False}},
        'assertions': {
            'exception': 'UnsupportedS3ControlConfigurationError',
        },
    },
    {
        'arn': 'arn:aws-cn:s3-outposts:cn-north-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'region': 'us-west-2',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'exception': 'UnsupportedS3ControlConfigurationError',
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'region': 'us-gov-east-1',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-gov-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'region': 'us-gov-east-1-fips',
        'config': {'s3': {'use_arn_region': False}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts-fips.us-gov-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:fips-us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'region': 'fips-us-gov-east-1',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'region': 'us-gov-east-1-fips',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts-fips.us-gov-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'config': {'s3': {'use_dualstack_endpoint': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-west-2.api.aws',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:accesspoint:myaccesspoint',
        'config': {'s3': {'use_accelerate_endpoint': True}},
        'assertions': {
            'exception': 'UnsupportedS3ControlConfigurationError',
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost',
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456',
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:myaccesspoint',
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
]


BUCKET_ARN_TEST_CASES = [
    # Outpost bucket arn test cases
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-west-2',
        'config': {},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-west-2.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-west-2',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-west-2',
        'config': {'s3': {'use_arn_region': False}},
        'assertions': {
            'exception': 'UnsupportedS3ControlConfigurationError',
        },
    },
    {
        'arn': 'arn:aws-cn:s3-outposts:cn-north-1:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-west-2',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'exception': 'UnsupportedS3ControlConfigurationError',
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-gov-east-1',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-gov-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-gov-east-1-fips',
        'config': {'s3': {'use_arn_region': False}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts-fips.us-gov-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:fips-us-gov-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'fips-us-gov-east-1',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
    {
        'arn': 'arn:aws-us-gov:s3-outposts:us-gov-east-1:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-gov-east-1-fips',
        'config': {'s3': {'use_arn_region': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts-fips.us-gov-east-1.amazonaws.com',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket:mybucket',
        'region': 'us-west-2',
        'config': {'s3': {'use_dualstack_endpoint': True}},
        'assertions': {
            'signing_name': 's3-outposts',
            'netloc': 's3-outposts.us-west-2.api.aws',
            'headers': {
                'x-amz-outpost-id': 'op-01234567890123456',
                'x-amz-account-id': '123456789012',
            },
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost',
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456',
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:bucket',
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
    {
        'arn': 'arn:aws:s3-outposts:us-west-2:123456789012:outpost:op-01234567890123456:bucket',
        'assertions': {
            'exception': 'UnsupportedS3ControlArnError',
        },
    },
]


V4_AUTH_REGEX = re.compile(
    r'AWS4-HMAC-SHA256 Credential=\w+/\d+/'
    r'(?P<region>[a-z0-9-]+)/(?P<name>[a-z0-9-]+)/'
)


def _assert_signing_name(stubber, expected_name):
    request = stubber.requests[0]
    auth_header = request.headers['Authorization'].decode('utf-8')
    actual_name = V4_AUTH_REGEX.match(auth_header).group('name')
    assert expected_name == actual_name


def _assert_netloc(stubber, expected_netloc):
    request = stubber.requests[0]
    url_parts = urlsplit(request.url)
    assert expected_netloc == url_parts.netloc


def _assert_header(stubber, key, value):
    request = stubber.requests[0]
    assert key in request.headers
    actual_value = request.headers[key]
    if isinstance(actual_value, bytes):
        actual_value = actual_value.decode('utf-8')
    assert value == actual_value


def _assert_headers(stubber, headers):
    for key, value in headers.items():
        _assert_header(stubber, key, value)


def _bootstrap_session():
    session = Session()
    session.set_credentials('access_key', 'secret_key')
    return session


def _bootstrap_client(session, region, **kwargs):
    client = session.create_client('s3control', region, **kwargs)
    stubber = ClientHTTPStubber(client)
    return client, stubber


def _bootstrap_test_case_client(session, test_case):
    region = test_case.get('region', 'us-west-2')
    config = test_case.get('config', {})
    config = Config(**config)
    return _bootstrap_client(session, region, config=config)


@pytest.mark.parametrize("test_case", ACCESSPOINT_ARN_TEST_CASES)
def test_accesspoint_arn_redirection(test_case):
    session = _bootstrap_session()
    client, stubber = _bootstrap_test_case_client(session, test_case)
    with _assert_test_case(test_case, client, stubber):
        client.get_access_point_policy(Name=test_case['arn'])


@pytest.mark.parametrize("test_case", BUCKET_ARN_TEST_CASES)
def test_bucket_arn_redirection(test_case):
    session = _bootstrap_session()
    client, stubber = _bootstrap_test_case_client(session, test_case)
    with _assert_test_case(test_case, client, stubber):
        client.get_bucket(Bucket=test_case['arn'])


@contextmanager
def _assert_test_case(test_case, client, stubber):
    stubber.add_response()
    assertions = test_case['assertions']
    exception_raised = None
    try:
        with stubber:
            yield
    except Exception as e:
        if 'exception' not in assertions:
            raise
        exception_raised = e
    if 'exception' in assertions:
        exception_cls = getattr(exceptions, assertions['exception'])
        if exception_raised is None:
            raise RuntimeError(
                'Expected exception "%s" was not raised' % exception_cls
            )
        error_msg = ('Expected exception "%s", got "%s"') % (
            exception_cls,
            type(exception_raised),
        )
        assert isinstance(exception_raised, exception_cls), error_msg
    else:
        assert len(stubber.requests) == 1
        if 'signing_name' in assertions:
            _assert_signing_name(stubber, assertions['signing_name'])
        if 'headers' in assertions:
            _assert_headers(stubber, assertions['headers'])
        if 'netloc' in assertions:
            _assert_netloc(stubber, assertions['netloc'])


class TestS3ControlRedirection(unittest.TestCase):
    def setUp(self):
        self.session = _bootstrap_session()
        self.region = 'us-west-2'
        self._bootstrap_client()

    def _bootstrap_client(self, **kwargs):
        client, stubber = _bootstrap_client(
            self.session, self.region, **kwargs
        )
        self.client = client
        self.stubber = stubber

    def test_outpost_id_redirection_dualstack(self):
        config = Config(s3={'use_dualstack_endpoint': True})
        self._bootstrap_client(config=config)
        self.stubber.add_response()
        with self.stubber:
            self.client.create_bucket(Bucket='foo', OutpostId='op-123')
        _assert_netloc(self.stubber, 's3-outposts.us-west-2.api.aws')
        _assert_header(self.stubber, 'x-amz-outpost-id', 'op-123')

    def test_outpost_id_redirection_create_bucket(self):
        self.stubber.add_response()
        with self.stubber:
            self.client.create_bucket(Bucket='foo', OutpostId='op-123')
        _assert_netloc(self.stubber, 's3-outposts.us-west-2.amazonaws.com')
        _assert_header(self.stubber, 'x-amz-outpost-id', 'op-123')

    def test_outpost_id_redirection_list_regional_buckets(self):
        self.stubber.add_response()
        with self.stubber:
            self.client.list_regional_buckets(
                OutpostId='op-123',
                AccountId='1234',
            )
        _assert_netloc(self.stubber, 's3-outposts.us-west-2.amazonaws.com')
        _assert_header(self.stubber, 'x-amz-outpost-id', 'op-123')

    def test_outpost_redirection_custom_endpoint(self):
        self._bootstrap_client(endpoint_url='https://outpost.foo.com/')
        self.stubber.add_response()
        with self.stubber:
            self.client.create_bucket(Bucket='foo', OutpostId='op-123')
        _assert_netloc(self.stubber, 'outpost.foo.com')
        _assert_header(self.stubber, 'x-amz-outpost-id', 'op-123')

    def test_normal_ap_request_has_correct_endpoint(self):
        self.stubber.add_response()
        with self.stubber:
            self.client.get_access_point_policy(Name='MyAp', AccountId='1234')
        _assert_netloc(self.stubber, '1234.s3-control.us-west-2.amazonaws.com')

    def test_normal_ap_request_custom_endpoint(self):
        self._bootstrap_client(endpoint_url='https://example.com/')
        self.stubber.add_response()
        with self.stubber:
            self.client.get_access_point_policy(Name='MyAp', AccountId='1234')
        _assert_netloc(self.stubber, '1234.example.com')

    def test_normal_bucket_request_has_correct_endpoint(self):
        self.stubber.add_response()
        with self.stubber:
            self.client.create_bucket(Bucket='foo')
        _assert_netloc(self.stubber, 's3-control.us-west-2.amazonaws.com')

    def test_arn_account_id_mismatch(self):
        arn = (
            'arn:aws:s3-outposts:us-west-2:123456789012:outpost:'
            'op-01234567890123456:accesspoint:myaccesspoint'
        )
        with self.assertRaises(UnsupportedS3ControlArnError):
            self.client.get_access_point_policy(Name=arn, AccountId='1234')

    def test_create_access_point_does_not_expand_name(self):
        arn = (
            'arn:aws:s3-outposts:us-west-2:123456789012:outpost:'
            'op-01234567890123456:accesspoint:myaccesspoint'
        )
        # AccountId will not be backfilled by the arn expansion and thus
        # fail parameter validation
        with self.assertRaises(ParamValidationError):
            self.client.create_access_point(Name=arn, Bucket='foo')

    def test_arn_invalid_host_label(self):
        config = Config(s3={'use_arn_region': True})
        self._bootstrap_client(config=config)
        arn = (
            'arn:aws:s3-outposts:us-we$t-2:123456789012:outpost:'
            'op-01234567890123456:accesspoint:myaccesspoint'
        )
        with self.assertRaises(InvalidHostLabelError):
            self.client.get_access_point_policy(Name=arn)

    def test_unknown_arn_format(self):
        arn = 'arn:aws:foo:us-west-2:123456789012:bar:myresource'
        with self.assertRaises(UnsupportedS3ControlArnError):
            self.client.get_access_point_policy(Name=arn)



# Copyright 2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import pytest

from tests import BaseSessionTest, ClientHTTPStubber


class BaseSQSOperationTest(BaseSessionTest):
    def setUp(self):
        super().setUp()
        self.region = "us-west-2"
        self.client = self.session.create_client("sqs", self.region)
        self.http_stubber = ClientHTTPStubber(self.client)


class SQSQueryCompatibleTest(BaseSQSOperationTest):
    def test_query_compatible_error_parsing(self):
        """When migrating SQS from the ``query`` protocol to ``json``,
        we unintentionally moved from modeled errors to a general ``ClientError``.
        This ensures we're not silently regressing that behavior.
        """

        error_body = (
            b'{"__type":"com.amazonaws.sqs#QueueDoesNotExist",'
            b'"message":"The specified queue does not exist."}'
        )
        error_headers = {
            "x-amzn-query-error": "AWS.SimpleQueueService.NonExistentQueue;Sender",
        }
        with self.http_stubber as stub:
            stub.add_response(
                status=400, body=error_body, headers=error_headers
            )
            with pytest.raises(self.client.exceptions.QueueDoesNotExist):
                self.client.delete_queue(
                    QueueUrl="not-a-real-queue-botocore",
                )



#!/usr/bin/env
# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import pickle

import botocore.awsrequest
import botocore.session
from botocore import exceptions
from tests import unittest


def test_client_error_can_handle_missing_code_or_message():
    response = {'Error': {}}
    expect = 'An error occurred (Unknown) when calling the blackhole operation: Unknown'
    assert str(exceptions.ClientError(response, 'blackhole')) == expect


def test_client_error_has_operation_name_set():
    response = {'Error': {}}
    exception = exceptions.ClientError(response, 'blackhole')
    assert hasattr(exception, 'operation_name')


def test_client_error_set_correct_operation_name():
    response = {'Error': {}}
    exception = exceptions.ClientError(response, 'blackhole')
    assert exception.operation_name == 'blackhole'


def test_retry_info_added_when_present():
    response = {
        'Error': {},
        'ResponseMetadata': {
            'MaxAttemptsReached': True,
            'RetryAttempts': 3,
        },
    }
    error_msg = str(exceptions.ClientError(response, 'operation'))
    if '(reached max retries: 3)' not in error_msg:
        raise AssertionError(
            "retry information not inject into error "
            "message: %s" % error_msg
        )


def test_retry_info_not_added_if_retry_attempts_not_present():
    response = {
        'Error': {},
        'ResponseMetadata': {
            'MaxAttemptsReached': True,
        },
    }
    # Because RetryAttempts is missing, retry info is not
    # in the error message.
    error_msg = str(exceptions.ClientError(response, 'operation'))
    if 'max retries' in error_msg:
        raise AssertionError(
            "Retry information should not be in exception "
            "message when retry attempts not in response "
            "metadata: %s" % error_msg
        )


def test_can_handle_when_response_missing_error_key():
    response = {
        'ResponseMetadata': {
            'HTTPHeaders': {},
            'HTTPStatusCode': 503,
            'MaxAttemptsReached': True,
            'RetryAttempts': 4,
        }
    }
    e = exceptions.ClientError(response, 'SomeOperation')
    if 'An error occurred (Unknown)' not in str(e):
        raise AssertionError(
            "Error code should default to 'Unknown' "
            "when missing error response, instead got: %s" % str(e)
        )


class TestPickleExceptions(unittest.TestCase):
    def test_single_kwarg_botocore_error(self):
        exception = botocore.exceptions.DataNotFoundError(data_path='mypath')
        unpickled_exception = pickle.loads(pickle.dumps(exception))
        self.assertIsInstance(
            unpickled_exception, botocore.exceptions.DataNotFoundError
        )
        self.assertEqual(str(unpickled_exception), str(exception))
        self.assertEqual(unpickled_exception.kwargs, exception.kwargs)

    def test_multiple_kwarg_botocore_error(self):
        exception = botocore.exceptions.UnknownServiceError(
            service_name='myservice', known_service_names=['s3']
        )
        unpickled_exception = pickle.loads(pickle.dumps(exception))
        self.assertIsInstance(
            unpickled_exception, botocore.exceptions.UnknownServiceError
        )
        self.assertEqual(str(unpickled_exception), str(exception))
        self.assertEqual(unpickled_exception.kwargs, exception.kwargs)

    def test_client_error(self):
        exception = botocore.exceptions.ClientError(
            error_response={
                'Error': {'Code': 'MyCode', 'Message': 'MyMessage'}
            },
            operation_name='myoperation',
        )
        unpickled_exception = pickle.loads(pickle.dumps(exception))
        self.assertIsInstance(
            unpickled_exception, botocore.exceptions.ClientError
        )
        self.assertEqual(str(unpickled_exception), str(exception))
        self.assertEqual(
            unpickled_exception.operation_name, exception.operation_name
        )
        self.assertEqual(unpickled_exception.response, exception.response)

    def test_dynamic_client_error(self):
        session = botocore.session.Session()
        client = session.create_client('s3', 'us-west-2')
        exception = client.exceptions.NoSuchKey(
            error_response={
                'Error': {'Code': 'NoSuchKey', 'Message': 'Not Found'}
            },
            operation_name='myoperation',
        )
        unpickled_exception = pickle.loads(pickle.dumps(exception))
        self.assertIsInstance(
            unpickled_exception, botocore.exceptions.ClientError
        )
        self.assertEqual(str(unpickled_exception), str(exception))
        self.assertEqual(
            unpickled_exception.operation_name, exception.operation_name
        )
        self.assertEqual(unpickled_exception.response, exception.response)

    def test_http_client_error(self):
        exception = botocore.exceptions.HTTPClientError(
            botocore.awsrequest.AWSRequest(),
            botocore.awsrequest.AWSResponse(
                url='https://foo.com', status_code=400, headers={}, raw=b''
            ),
            error='error',
        )
        unpickled_exception = pickle.loads(pickle.dumps(exception))
        self.assertIsInstance(
            unpickled_exception, botocore.exceptions.HTTPClientError
        )
        self.assertEqual(str(unpickled_exception), str(exception))
        self.assertEqual(unpickled_exception.kwargs, exception.kwargs)
        # The request/response properties on the HTTPClientError do not have
        # __eq__ defined so we want to make sure properties are at least
        # of the expected type
        self.assertIsInstance(
            unpickled_exception.request, botocore.awsrequest.AWSRequest
        )
        self.assertIsInstance(
            unpickled_exception.response, botocore.awsrequest.AWSResponse
        )



import pytest

import botocore
from tests import FreezeTime, requires_crt
from tests.unit.auth.test_sigv4 import (
    DATE,
    REGION,
    SERVICE,
    SignatureTestCase,
    assert_equal,
    create_request_from_raw_request,
    generate_test_cases,
)


def _test_crt_signature_version_4(test_case):
    test_case = SignatureTestCase(test_case)
    request = create_request_from_raw_request(test_case.raw_request)

    # Use CRT logging to diagnose interim steps (canonical request, etc)
    # import awscrt.io
    # awscrt.io.init_logging(awscrt.io.LogLevel.Trace, 'stdout')
    auth = botocore.crt.auth.CrtSigV4Auth(
        test_case.credentials, SERVICE, REGION
    )
    auth.add_auth(request)
    actual_auth_header = request.headers["Authorization"]
    assert_equal(
        actual_auth_header,
        test_case.authorization_header,
        test_case.raw_request,
        "authheader",
    )


@requires_crt()
@pytest.mark.parametrize("test_case", generate_test_cases())
@FreezeTime(module=botocore.auth.datetime, date=DATE)
def test_signature_version_4(test_case):
    _test_crt_signature_version_4(test_case)
